// le20c.h - LE20c QBUS hosted SUMMA interface by Vitya Yakimchuk
//
// Mar-2004 by Solo, adapted from Serge sources
//

#ifndef _LE20C_H
#define _LE20C_H

#include "qbus.h"

#define LE20C_COM0_PORT 	0x0
#define LE20C_COM1_PORT		0x2	
#define LE20C_DATA_PORT		0x8
#define LE20C_STATUS_PORT	0xa
#define LE20C_SYNC_PORT		0xa

class LE20C
{
  
 public:
  LE20C (QBUS* host, int c = 0)
    {
      base_port = 0xff00 + c * 0x10;
      qbus = host;
      crate = c;
      //      zero ();
    }

  ~LE20C () {}

//  Front panel NIM pulse
  void sync ()  {  qbus->write (base_port+LE20C_SYNC_PORT, 1);  }
//  N(28)A(8)F(26) - Zero, N(28)A(9)F(26) - Clear
  void zero  () {  qbus->write (base_port+LE20C_COM0_PORT, command (28, 8, 26));  }
  void clear () {  qbus->write (base_port+LE20C_COM0_PORT, command (28, 9, 26));  }
  void Z ()     {  zero ();  }
  void C ()     {  clear ();  }
  short X ()    {  return x; }
  short Q ()    {  return q; }
  void status (){  get_status(); }
  void print () {  get_status(); printf("SUMMA crate %d status Q=%x X=%x\n",crate,q,x); }
  int  number (){  return crate; }
  QBUS * host (){  return qbus; }

//  LAM read is not yet implemented
  short lam   ()
  {
    return 0;
  }

  void write (int n, int a, short data) {
    qbus->write (base_port+LE20C_DATA_PORT, data);
    qbus->write (base_port+LE20C_COM0_PORT, command (n, a, 0x10));
    get_status();
  }

  short read (int n, int a) {
    short data;
    qbus->write (base_port+LE20C_COM0_PORT, command (n, a, 0));
    data = qbus->read (base_port+LE20C_DATA_PORT);
    get_status();
    return data;
  }

  short function (int n, int a, int f, short data = 0) {
    if (f > 0xf)
      {
	qbus->write (base_port+LE20C_DATA_PORT, data);
	qbus->write (base_port+LE20C_COM0_PORT, command (n, a, f));
      }
    else
      {
	qbus->write (base_port+LE20C_COM0_PORT, command (n, a, f));
	data = qbus->read (base_port+LE20C_DATA_PORT);
      }
    get_status();
    return data;
  }

  void act (int n, int a, int f) {
    qbus->write (base_port+LE20C_COM0_PORT, command (n, a, f));
    get_status();
  }

 private:
  int base_port, crate;
  QBUS* qbus;
//  CAMAC X and Q signals
  int x, q;

  short com_0_port ()   {  return base_port;  }
  short com_1_port ()   {  return base_port + 2;  }
  short data_port ()    {  return base_port + 8;  }
  short status_port ()  {  return base_port + 0xa;  }
  short sync_port ()    {  return base_port + 0xa;  }

  short command (int n, int a, int f)
  {
    return 0x4000 + ((f & 0x1f) << 9) + ((n & 0x1f) << 4) + (a & 0xf);
  }

  void get_status() {
    short status;    
    status = qbus->read (base_port+LE20C_COM1_PORT);
    q = ((status & 1) == 1);
    x = ((status & 2) == 2);
  }

};

#endif
