// le71.h - LE-71 96 channel MISS ADC digital part (replaces LE-19)
//
// Nov-2006 by Solo
//
#ifndef _LE71_H
#define _LE71_H

#include "miss.h"

#define LE71_MAX_MODULE   4
#define LE71_MAX_CHANNEL  96
#define LE71_MAX_CHANNELS (LE71_MAX_MODULE*LE71_MAX_CHANNEL)
#define LE71_MAX_PED      4096

#define LE71_CTRL         0
#define LE71_ADDR         1
#define LE71_DATA         2
#define LE71_PED          3

#define LE71_INCR         1  // does not work
#define LE71_TEST         2
#define LE71_INTERNAL     4  // does not work

class LE71
{

 private:

  MISS* h; // MISS host
  int mn;  // module logical number
  int n;   // MISS module number

 public:

  LE71 (MISS* host, int number)
  {
    h = host;
    mn = number;
    n = mn<<1; // due to big number of channels, miss module number is shifted
  }

  ~LE71 () {};

  // write address
  int waddr(int addr) {
    h->write(n,LE71_ADDR,addr);
    return addr;
  }

  // read address
  int raddr() {
    return h->read(n,LE71_ADDR);
  }

  // read pedestals/data from module memory
  int read (short *data, int peds=1) {
    int ctrl,reg;

    if (peds) {
      ctrl=0;
      reg=LE71_PED;
    }
    else {
      ctrl=LE71_TEST;
      reg=LE71_DATA;
    }

    h->clear();                 // clear AR mode and reset bus
    h->write(n,LE71_CTRL,ctrl); // set control register

    for(int i=0; i<LE71_MAX_CHANNEL; i++) {
      h->write(n,LE71_ADDR,i);
      data[i]=h->read(n,reg);
    }

    return LE71_MAX_CHANNEL;
  }

  // write pedestals/data to module memory
  int write (short *data, int peds=1) {
    int ctrl,reg;

    if (peds) {
      ctrl=0;
      reg=LE71_PED;
    }
    else {
      ctrl=LE71_TEST;
      reg=LE71_DATA;
    }

    h->clear();                 // clear AR mode and reset bus
    h->write(n,LE71_CTRL,ctrl); // clear control register

    for(int i=0; i<LE71_MAX_CHANNEL; i++) {
      h->write(n,LE71_ADDR,i);
      h->write(n,reg,data[i]);
    }

    return LE71_MAX_CHANNEL;
  }

  // clear pedestals/data in module memory
  int clear (int peds=1) {
    int ctrl,reg;

    if (peds) {
      ctrl=0;
      reg=LE71_PED;
    }
    else {
      ctrl=LE71_TEST;
      reg=LE71_DATA;
    }

    h->clear();                 // clear AR mode and reset bus
    h->write(n,LE71_CTRL,ctrl); // clear control register

    for(int i=0; i<LE71_MAX_CHANNEL; i++) {
      h->write(n,LE71_ADDR,i);
      h->write(n,reg,0);
    }

    return LE71_MAX_CHANNEL;
  }

  // refresh pedestals
  int refresh(short *ped) {
    short buf[LE71_MAX_CHANNEL];
    int r=0;

    read(buf);
    for(int i=0; i<LE71_MAX_CHANNEL; i++) {
      if(buf[i]!=ped[i]) r++;
    }
    write(ped);
    return r;    
  }

};

# endif
