// le83.h - LE83 fast QBUS hosted MISS interface by Vitya Yakimchuk
//
// May-2007 by Solo
//

#ifndef _LE83_H
#define _LE83_H

#include "miss.h"

class LE83: public MISS
{

 public:
  LE83 (QBUS* host, int c = 0): MISS (host, c)
  {
    //    au ();
    //    clear ();
  }

  virtual ~LE83() {}

//  MISS operations
  virtual void sync ()  {  qbus->write (sync_port (), 0);  } // Send pulse to NIM output
  virtual void addr ()  {  set_mode (0x3f & ur);  }          // Set address mode
  virtual void au ()    {  set_mode (0xc0 | (0x3f & ur));  } // Set autonomous mode
  virtual void clear () {  set_command (0x8000); set_command (0x8000);  } // Send Clear to crate
  virtual short status () { return qbus->read (stat_port ()); }

  virtual int tr ()  {  return ((status () & 0x10) > 0);  }
  virtual int gt ()  {
    if(use_gt) return ((status () & 8) > 0); 
    else return 0; 
  }
  virtual int bus_error ()  {  return ((get_num () & 0x8000) > 0);  }

  virtual void print() {
    static const char *status_name[16]={"T1","T2","T3",
					"GT - ready",
					"TR - active request",
					"ERR - error",
					"PCHN - serial numbers read",
					"PCHI - serial data read",
					"AR - autonomous mode",
					"SI1 - synch. signal 1",
					"SI2 - synch. signal 2",
					"","","","",
                                        "ND - new data"};
    unsigned short s=(status()&0xffff);
    printf("MISS crate %d status=0x%04x\n",crate,s); 
    for(int i=0; i<16; i++) {
      if((s&0x0001) == 1) printf("0x%04x = %s\n",1<<i,status_name[i]);
      s=s>>1;
    }
  }

  virtual short test (short w) {
    set_data(w);
    return get_data();
  }

  virtual void write (int n, int a, short data) {
    addr ();
    set_data (data);
    set_command (command (n, a, 3));              // Write function 3
    if (bus_error ()) error = bad_address;
    else              error = none;    
  }

  virtual short read (int n, int a) {
    short data;
    addr ();
    set_command (command (n, a, 0));              // Read function 0!
    data = get_data ();
    if (bus_error ()) error = bad_address;
    else              error = none;
    return data;
  }

  virtual short function (int n, int a, int f, short data = 0) {
    addr ();
    if ((f == 3) or (f == 5)) set_data (data);
    set_command (command (n, a, f));
    if ((f == 0) or (f == 1) or (f == 4)) data = get_data ();
    if (bus_error ()) error = bad_address;
    else              error = none;
    return data;
  }

  virtual int sdata (int maxsize, short* data, short* num = NULL) {
    int tt=0;

    // wait for GT to disappear
    while (gt ())      
      if (tt++ > timeout)
	{
	  error = gt_timeout;
	  return -1;
	}

    set_mode (0x80 + ur);       //  Set PCHI mode

    int cnt = 0;
    while (nd ())
      {
	if (cnt++ >= maxsize)
	  {
	    error = buffer_overflow;
	    return maxsize;
	  }
	*(data++) = get_data ();
	if (num != NULL) *(num++) = get_num ();
	else                        get_num ();
      }

    set_mode(ur);                // set normal mode

    error = none;
    return cnt;
  }

  virtual int snumber (int maxsize, short* num) {
    set_mode (0x40 + ur);       //  Set PCHN mode

    int cnt = 0;
    while (nd ())
      {
	if (cnt++ >= maxsize)
	  {
	    error = buffer_overflow;
	    return maxsize;
	  }
	*(num++) = get_num ();
      }
    set_mode(ur);                // set normal mode

    error = none;
    return cnt;
  }

  virtual int sdata_fast16 (int maxsize, short* data, short* num = NULL) {
    int tt=0;

    // wait for GT to disappear
    while (gt ())      
      if (tt++ > timeout)
	{
	  error = gt_timeout;
	  return -1;
	}

    set_mode (0x80 + ur);       //  Set PCHI mode

    int cnt = 0;
    while (nd ())
      {
	for (int i=0; i<16; i++) {
	  if (cnt++ >= maxsize)
	    {
	      error = buffer_overflow;
	      return maxsize;
	    }
	  *(data++) = get_data ();
	  if (num != NULL) *(num++) = get_num ();
	  else                        get_num ();
	}
      }

    set_mode(ur);                // set normal mode

    error = none;
    return cnt;
  }

  virtual int sdata_fast (int maxsize, short* data, short* num = NULL) {
    int tt=0;
    int sync=0;

    // wait for GT to disappear
    while (gt ())      
      if (tt++ > timeout)
	{
	  error = gt_timeout;
	  return -1;
	}

    set_mode (0x180 + ur);       //  Set fast PCHI mode

    // wait for PCHI to end
    tt=0;
    while (!fpchi())
      if (tt++ > timeout)
	{
	  error = gt_timeout;
	  return -1;
	}

    // get number of words
    sync = get_sync();
    if (sync<0) return sync;

    int cnt = sync & 0x3ff;
    
    for(int i=0; i<cnt; i++)
      {
	if (i>maxsize) {
	  error = buffer_overflow;
	  return maxsize;
	}

	if (num != NULL) *(num++) = get_faddr ();
	else                        get_faddr ();
	*(data++) = get_fdata ();
      }

    set_mode(ur);                // set normal mode

    error = none;
    return cnt;
  }


 private:

// Private macros
  short stat_port ()    {  return base_port;  }
  short com_port ()     {  return base_port + 2;  }
  short mask_port ()    {  return base_port + 4;  }
  short mode_port ()    {  return base_port + 6;  }
  short data_port ()    {  return base_port + 8;  }
  short sync_port ()    {  return base_port + 0xa; }
  short faddr_port()    {  return base_port + 0xc; }
  short fdata_port()    {  return base_port + 0xe; }

  void set_data (short data)   {  qbus->write (data_port (), data);  }
  short get_data ()            {  return qbus->read (data_port ());  }
  short get_num ()             {  return qbus->read (com_port ());   }
  short get_sync ()            {  return qbus->read (sync_port ());  }
  short get_faddr ()           {  return qbus->read (faddr_port ()); }
  short get_fdata ()           {  return qbus->read (fdata_port ()); } 
  void set_command (short cmd) {  qbus->write (com_port (), cmd);    }
  void set_mode (short mode)   {  qbus->write (mode_port (), mode);  }

  int nd ()  {  return ((status () & 0x8000) > 0);  }
  int fpchi () { return ((status () & 0x0800) > 0); }

  short command (int n, int a, int f)
  {
    return ((f & 7) << 12) + ((n & 0x3f) << 6) + (a & 0x3f);
  }
};

#endif
