// miss.h - MISS interface class
//
// Mar-2004 by Solo, adapted from Serge sources
//

#ifndef _MISS_H
#define _MISS_H

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>

#include "qbus.h" // PC-QBUS card

class MISS
{

 public:
  MISS() {}
  MISS (QBUS* host, int c = 0)
  {
    crate = c;
    base_port = 0xfe00 + c * 0x10;
    qbus = host;
    error = none;
    ur = 0;
    timeout = 1000;
    use_gt = true;
  }

 virtual ~MISS() {}

//  MISS virtual functions
  virtual void sync()  {} //  Send pulse to NIM output
  virtual void addr()  {} //  Set address mode
  virtual void au()    {} //  Set autonomous mode
  virtual void clear() {} //  Send Clear to crate
  virtual void Z()     { clear (); } //  The same as "clear"
  virtual short status() { return 0; } // Get status
  virtual void print() {} // Print Status

  virtual int tr ()  {  return 0; }
  virtual int gt ()  {  return 0;  }
  virtual int bus_error ()  {  return 0; }

  virtual void write(int n, int a, short data) {}
  virtual short read(int n, int a) { return 0; }
  virtual short function(int n, int a, int f, short data = 0) { return 0; }
  virtual short test(short word) { return -1; }

  virtual int sdata(int maxsize, short* data, short* num = NULL) { return 0; }
  virtual int sdata_fast(int maxsize, short* data, short* num = NULL) { return 0; }
  virtual int sdata_fast16 (int maxsize, short* data, short* num = NULL) { return 0; }
  virtual int snumber(int maxsize, short* num) { return 0; }

  int crate;
//  Registration control lines
  int ur;
  float timeout;
  bool use_gt;

//  MISS errors
  enum error_type
  {
    none,
    bad_address,
    buffer_overflow,
    data_timeout,
    gt_timeout
  } error;

  virtual char* error_string() {
    char suffix[40];
    switch (error)
      {
      case none:            strcpy (error_string_buffer, ""); break;
      case bad_address:     strcpy (suffix, "bad address"); break;
      case buffer_overflow: strcpy (suffix, "data buffer overflow"); break;
      case data_timeout:    strcpy (suffix, "timeout waiting data word"); break;
      case gt_timeout:      strcpy (suffix, "timeout waiting GT"); break;
      default:              strcpy (suffix, "unknown error"); 
      }
    sprintf (error_string_buffer, "Error in MISS crate %d: %s", crate, suffix);

    return error_string_buffer;
  }

 protected:
  int base_port;
  QBUS* qbus;
  char error_string_buffer[80];
};

#endif
