#include "le90m_widget.h"

static char *le90m_programmable_parameter_name[]={"1 TSSCOPE",
						  "2 TBSGATE",
						  "3 TLED",
						  "4 TGATEES",
						  "5 TINHIBIT",
						  "6 TESSTART",
						  "7 FLED",
						  "8 FTEST"};

static char *le90m_programmable_parameter_unit[]={"s","s","s","s","s","s","Hz","kHz"};

LE90Mwidget::LE90Mwidget(LE90M *le, QWidget *parent) : QWidget(parent)
{
  // copy device pointer
  module = le;

  // create top level layout and box
  QVBoxLayout *mylayout = new QVBoxLayout;
  QGroupBox *mygroupbox = new QGroupBox("LE90M control");
  QVBoxLayout *layout = new QVBoxLayout;
  mygroupbox->setLayout(layout);
  mylayout->addWidget(mygroupbox);

  // add setup parameters
  QGroupBox *sgbox = new QGroupBox("Setup paramaters");
  QVBoxLayout *svbox = new QVBoxLayout;
  setup_parameter[0] = new LE90Msetup("QBUS device",le->host()->host()->device());
  setup_parameter[1] = new LE90Msetup("SUMMA crate",QString::number(le->host()->number()));
  setup_parameter[2] = new LE90Msetup("SUMMA module",QString::number(le->number()));
  svbox->addWidget(setup_parameter[0]);
  svbox->addWidget(setup_parameter[1]);
  svbox->addWidget(setup_parameter[2]);
  sgbox->setLayout(svbox);
  layout->addWidget(sgbox);

  // add enable/disable font panel programming
  QGroupBox *fgbox = new QGroupBox("Front panel programming");
  QHBoxLayout *fhbox = new QHBoxLayout;
  QRadioButton *fbenabled = new QRadioButton("Enabled");
  QRadioButton *fbdisabled = new QRadioButton("Disabled");
  fhbox->addWidget(fbenabled);
  fhbox->addWidget(fbdisabled);
  fgbox->setLayout(fhbox);
  layout->addWidget(fgbox);

  // add manual parameters 
  QGroupBox *mgbox = new QGroupBox("Manual parameters");
  QVBoxLayout *mvbox = new QVBoxLayout;
  manual_parameter[0] = new LE90Mparameter("TBS","s",false);
  manual_parameter[1] = new LE90Mparameter("TGATE","s",false);
  mvbox->addWidget(manual_parameter[0]);
  mvbox->addWidget(manual_parameter[1]);
  mgbox->setLayout(mvbox);
  layout->addWidget(mgbox);

  // add programmable parameters
  QGroupBox *pgbox = new QGroupBox("Programmable parameters");
  QVBoxLayout *pvbox = new QVBoxLayout;
  for(unsigned int i=0; i<8; i++) {
    programmable_parameter[i]=new LE90Mparameter(le90m_programmable_parameter_name[i],
						 le90m_programmable_parameter_unit[i],
						 true);
    pvbox->addWidget(programmable_parameter[i]);
  }
  pgbox->setLayout(pvbox);
  layout->addWidget(pgbox);

  // add update button
  QPushButton *bupdate = new QPushButton("&Update");
  layout->addWidget(bupdate);

  setLayout(mylayout);

  // connect update button
  QObject::connect(bupdate, SIGNAL(clicked()), this, SLOT(updateLE90M(void)));

  // update it
  updateLE90M();

}

void LE90Mwidget::updateLE90M(void)
{
  //  printf("Aga!\n");
  //  module->print();
  LE90MDATA data;
  module->read(&data);
  manual_parameter[0]->displayParameter(data.tbs/10.0);
  manual_parameter[1]->displayParameter(data.tgate/10.0);

  programmable_parameter[0]->displayParameter(data.tsscope/10.0);
  programmable_parameter[1]->displayParameter(data.tbsgate/10.0);
  programmable_parameter[2]->displayParameter(data.tled/10.0);
  programmable_parameter[3]->displayParameter(data.tgatees/10.0);
  programmable_parameter[4]->displayParameter(data.tinhibit/10.0);
  programmable_parameter[5]->displayParameter(data.tesstart/10.0);
  programmable_parameter[6]->displayParameter(data.fled/10.0);
  programmable_parameter[7]->displayParameter(data.ftest/10.0);
  
}
