// le90mqt.cc - Qt GUI interface for le90m module
//
// Nov-2007 by Solo

#include <QApplication>
#include <QVBoxLayout>
#include <QPushButton>
#include <QLabel>
#include <QTimer>
#include "timer.h"
#include "le90m_widget.h"

int main(int argc, char *argv[])
{
  QBUS *qu;
  LE20C *su;
  LE90M *le;

  // create things
  if (argc>1) qu = new QBUS(argv[1]);
  else qu = new QBUS("/dev/pq2");
  if (argc>2) su = new LE20C(qu,atoi(argv[2]));
  else su = new LE20C(qu,4);
  if (argc>3) le = new LE90M(su,atoi(argv[3]));
  else le = new LE90M(su,4);

  QApplication app(argc, argv);
  QWidget win;
  QVBoxLayout vbox;
  Timer datetime;
  LE90Mwidget widget(le);
  QPushButton quit("&Quit");

  // lay out stuff
  vbox.setContentsMargins(0,0,0,0);
  vbox.addWidget(&datetime);
  vbox.addWidget(&widget);
  vbox.addWidget(&quit);
  win.setLayout(&vbox);

  // connect quit button
  QObject::connect(&quit, SIGNAL(clicked()), &app, SLOT(quit()));

  // create and connect update timer
  QTimer *tm = new QTimer(&app);
  QObject::connect(tm, SIGNAL(timeout()), &widget, SLOT(readLE90M()));
  QObject::connect(tm, SIGNAL(timeout()), &datetime, SLOT(updateTime()));
  tm->start(3000);

  // run the show
  win.show();
  return app.exec();
}
