// tl2_widget.cpp - TL2-50 trigger control widget 
//
// Nov-2008 by Solo
//

#include "tl2_widget.h"

#include <QFileDialog>
#include <QSignalMapper>
#include <QTimer>

TL2widget::TL2widget(TL2 *tl, QWidget *parent) : QWidget(parent)
{
  // set default names
  char input_names[16][10]={"S1","S2","S3","S4","S10","S11","K1K2","A10","A11","","","","","","",""};

  // copy device pointer
  module = tl;

  // set smaller font
  setStyleSheet("font-size: 8pt");

  // create top level layout and box
  QVBoxLayout *mylayout = new QVBoxLayout;
  QGroupBox *mygroupbox = new QGroupBox("TL2-50 control");
  QHBoxLayout *layout = new QHBoxLayout;
  mygroupbox->setLayout(layout);
  mylayout->addWidget(mygroupbox);

  // create left control pad
  QVBoxLayout *leftlayout = new QVBoxLayout;
  QGroupBox *leftgroupbox = new QGroupBox("General");
  leftgroupbox->setLayout(leftlayout);
  layout->addWidget(leftgroupbox);

  // add setup parameters
  QGroupBox *sgbox = new QGroupBox("Module");
  QVBoxLayout *shbox = new QVBoxLayout;
  setup_parameter[0] = new TL2setup("QBUS device",tl->host()->host()->device());
  setup_parameter[1] = new TL2setup("SUMMA crate",QString::number(tl->host()->number()));
  setup_parameter[2] = new TL2setup("SUMMA module",QString::number(tl->number()));
  shbox->addWidget(setup_parameter[0]);
  shbox->addWidget(setup_parameter[1]);
  shbox->addWidget(setup_parameter[2]);
  sgbox->setLayout(shbox);
  leftlayout->addWidget(sgbox);

  // trigger panel
  QGroupBox *tgbox = new QGroupBox("Trigger");
  QGridLayout *ggbox = new QGridLayout;
  //ggbox->setHorizontalSpacing(ggbox->horizontalSpacing()/2);
  ggbox->setVerticalSpacing(ggbox->verticalSpacing()/2);

  // row headers and trailers
  row_header[0] = new QLabel("Inputs");
  ggbox->addWidget(row_header[0],0,1);
  for(int i=0; i<4; i++) {
    trigger[i] = new QCheckBox("T"+QString::number(i+1));
    ggbox->addWidget(trigger[i],0,i+2,Qt::AlignCenter);
    triggers[i] = new QLineEdit();
    triggers[i]->setReadOnly(true);
    triggers[i]->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
    triggers[i]->setStyleSheet("padding: 0 0px");
    ggbox->addWidget(triggers[i],16+1,i+2);
    busy[i] = new QLineEdit();
    busy[i]->setReadOnly(true);
    busy[i]->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
    busy[i]->setStyleSheet("padding: 0 0px");
    ggbox->addWidget(busy[i],17+1,i+2);
  }
  row_header[4] = new QLabel("Counts");
  ggbox->addWidget(row_header[4],0,4+2);
  triggers[4] = new QLineEdit();
  triggers[4]->setReadOnly(true);
  triggers[4]->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
  triggers[4]->setStyleSheet("background-color: lightgreen; padding: 0 0px");
  ggbox->addWidget(triggers[4],17,4+2);
  busy[4] = new QLineEdit();
  busy[4]->setReadOnly(true);
  busy[4]->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
  busy[4]->setStyleSheet("background-color: lightyellow; padding: 0 0px");
  ggbox->addWidget(busy[4],18,4+2);

  // column headers and trailers
  for(int j=0; j<16; j++) {
    col_header[j] = new QLabel(QString::number(j+1));
    ggbox->addWidget(col_header[j],j+1,0);
    inputs[j] = new QLineEdit();
    inputs[j]->setStyleSheet("padding: 0 0px");
    inputs[j]->setText(QString(input_names[j]));
    ggbox->addWidget(inputs[j],j+1,1);
    counts[j] = new QLineEdit();
    counts[j]->setReadOnly(true);
    counts[j]->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
    counts[j]->setStyleSheet("padding: 0 0px");
    ggbox->addWidget(counts[j],j+1,6);
  }
  col_header[16] = new QLabel("All");
  ggbox->addWidget(col_header[16],16+1,0);
  col_header[17] = new QLabel("Busy");
  ggbox->addWidget(col_header[17],17+1,0);

  // trigger mask buttons
  for(int i=0; i<4; i++) {
    for(int j=0; j<16; j++) {
      input_mask[i][j]=new TL2mask();
      ggbox->addWidget(input_mask[i][j],j+1,i+2);
    }
  }
  tgbox->setLayout(ggbox);
  layout->addWidget(tgbox); 

  // add prescalers
  QGroupBox *pgbox = new QGroupBox("Prescalers");
  QVBoxLayout *phbox = new QVBoxLayout;
  for(int i=0; i<2; i++) {
    psc[i] = new QSpinBox();
    psc[i]->setRange(0,255);
    phbox->addWidget(psc[i]);
  }
  pgbox->setLayout(phbox);
  leftlayout->addWidget(pgbox);

  // add busy setup
  QGroupBox *bgbox = new QGroupBox("Busy");
  QVBoxLayout *bhbox = new QVBoxLayout;
  benabled = new QCheckBox("Auto Busy");
  benabled->setToolTip("Enable/Disable automatic busy after each trigger");
  bstatus = new QCheckBox("Busy status");
  bstatus->setEnabled(false); // status
  bhbox->addWidget(benabled);
  bhbox->addWidget(bstatus);
  bgbox->setLayout(bhbox);
  leftlayout->addWidget(bgbox);

  // add commands
  QGroupBox *cgbox = new QGroupBox("Commands");
  QVBoxLayout *chbox = new QVBoxLayout;
  QPushButton *breset = new QPushButton("Reset");
  breset->setToolTip("Reset all counters");
  //  QPushButton *bclear = new QPushButton("Clear");
  QPushButton *bload = new QPushButton("Load...");
  bload->setToolTip("Load configuration from file");
  QPushButton *bsave = new QPushButton("Save...");
  bsave->setToolTip("Save configuration to file");
  //  chbox->addWidget(bclear);
  chbox->addWidget(breset);
  chbox->addWidget(bload);
  chbox->addWidget(bsave);
  cgbox->setLayout(chbox);
  leftlayout->addWidget(cgbox);

  // add intensity plot
  for(int i=0; i<2048; i++) {
    intx[i]=i;
    inty[i]=0.0; 
  }
  intensityPlot =  new QwtPlot(QwtText("Beam intensity profile"));
  intensityCurve = new QwtPlotCurve("Intensity");
  intensityCurve->setRawData(intx,inty,2048);
  intensityCurve->setXAxis(QwtPlot::xTop);
  intensityCurve->attach(intensityPlot);
  intensityPlot->setAxisScale(QwtPlot::xBottom,0,2047*(1.7/1532.0));
  //  intensityPlot->setAxisTitle(QwtPlot::xBottom,"Time (s)");
  //  intensityPlot->enableAxis(QwtPlot::xTop);
  intensityPlot->setAxisScale(QwtPlot::xTop,0,2047);
  intensityPlot->setMinimumHeight(120);
  intensityZoomer = new QwtPlotZoomer(intensityPlot->canvas());
  //  intensityMagnifier = new QwtPlotMagnifier(intensityPlot->canvas());

  mylayout->addWidget(intensityPlot);

  setLayout(mylayout);

  // connect trigger and mask changes
  QSignalMapper *triggerMapper = new QSignalMapper(this);
  QSignalMapper *maskMapper = new QSignalMapper(this);
  for(int i=0; i<4; i++) {
    triggerMapper->setMapping(trigger[i],i);
    connect(trigger[i], SIGNAL(clicked()), triggerMapper, SLOT(map()));
    for(int j=0; j<16; j++) {
      maskMapper->setMapping(input_mask[i][j],i*16+j);
      connect(input_mask[i][j], SIGNAL(clicked()), maskMapper, SLOT(map()));
    }    
  }
  connect(triggerMapper, SIGNAL(mapped(int)), this, SLOT(changeTrigger(int)));
  connect(maskMapper, SIGNAL(mapped(int)), this, SLOT(changeMask(int)));

  // connect prescalers
  QSignalMapper *pscMapper = new QSignalMapper(this);
  for(int i=0; i<2; i++) {
    pscMapper->setMapping(psc[i],i);
    connect(psc[i], SIGNAL(valueChanged(int)), pscMapper, SLOT(map()));  
  }
  connect(pscMapper, SIGNAL(mapped(int)), this, SLOT(changePrescaler(int)));

  // connect buttons
  connect(breset, SIGNAL(clicked()), this, SLOT(reset(void)));
  connect(bload, SIGNAL(clicked()), this, SLOT(loadFile(void)));
  connect(bsave, SIGNAL(clicked()), this, SLOT(saveFile(void)));
  connect(benabled, SIGNAL(clicked()), this, SLOT(changeBusy(void)));

  // create and connect update timer
  QTimer *timer = new QTimer(this);
  connect(timer, SIGNAL(timeout()), this, SLOT(readTL2()));
  timer->start(3000);

  // trick to set default intensimeter frequency divider
  module->writefreq(2);

  // trick to set up mask for intensimeter
  module->writemask(3,0x30000);

  // read the stuff
  readTL2();

  // start zoomer
  intensityZoomer->zoom(0);

}

void TL2widget::reset() {
  module->reset();
  readTL2();
}

void TL2widget::readTL2() {
  // check for end of spill
  int c = module->counter(0);
  int d = c-cntSpill;

  if(d>0) isSpill=true;

  //  printf("%d %d %d\n",c,d,isSpill);

  if (d==0 && isSpill) {      
    // debug printout
    module->print();

    // read input counters
    for(int j=0; j<16; j++) {
      int cnt = module->counter(j);
      counts[j]->setText(QString::number(cnt));
      //    if(cnt>(1<<28)) counts[j]->setStyleSheet("background-color: red"); // overflow
    } 

    // read trigger all and busy counters
    for(int i=0; i<4; i++) {
      triggers[i]->setText(QString::number(module->trigall(i)));
      busy[i]->setText(QString::number(module->trigbusy(i)));
    }
    triggers[4]->setText(QString::number(module->trigsum()));
    busy[4]->setText(QString::number(module->busysum()));

    //read intensimeter
    for(int i=0; i<2048; i++) {
      //      printf("int[%d]=%d\n",i,module->intensity());
      inty[i]=module->intensity();
    }
    intensityPlot->replot();
    intensityZoomer->setZoomBase();

    isSpill = false;
    module->reset();
  }

  // read prescalers
  for(int i=0; i<2; i++) {
    psc[i]->setValue(module->readpsc(i));
  }

  // read masking
  int trig = module->readtrigmask();
  for(int i=0; i<4; i++) {
    if(((trig>>i)&1)==0) trigger[i]->setCheckState(Qt::Checked);
    else trigger[i]->setCheckState(Qt::Unchecked);

    int mask = module->readmask(i);
    int m = 0;
    for(int j=0; j<16; j++) {
      m = ((mask>>(16+j))&1)*2 + ((mask>>j)&1);
      if(m==1) input_mask[i][j]->setMask("A");
      else if (m==2) input_mask[i][j]->setMask("C");
      else input_mask[i][j]->setMask("O");
    }
  }

  // read busy setup and status
  int b = module->readbusy();
  if((b&1)==1) benabled->setCheckState(Qt::Checked);
  else benabled->setCheckState(Qt::Unchecked);

  if((b&2)==2) bstatus->setCheckState(Qt::Checked); 
  else bstatus->setCheckState(Qt::Unchecked);

  cntSpill = c;

}

void TL2widget::changePrescaler(int i) {
  module->writepsc(i,psc[i]->value());
}

void TL2widget::changeTrigger(int i) {
  int trig;

  trig = module->readtrigmask();
  if(trigger[i]->isChecked()) trig = trig&(~(1<<i));
  else trig = trig | (1<<i);
  module->writetrigmask(trig);
}

void TL2widget::changeMask(int i) {
  int trig=i/16;
  int chan=i%16;
  unsigned int mc;
  unsigned int ma;

  unsigned int mask = module->readmask(trig);
  ma = mask&0xffff;
  mc = (mask>>16)&0xffff;

  QString t = input_mask[trig][chan]->text();

  if(t=="C") {
    mc = mc|(1<<chan);
    ma = ma&(~(1<<chan));
  }
  else if (t=="A") {
    ma = ma|(1<<chan);
    mc = mc&(~(1<<chan));
  }
  else {
    mc = mc&(~(1<<chan));
    ma = ma&(~(1<<chan));
  }
  mask = (mc<<16)+ma;
  module->writemask(trig, mask);

  readTL2();
 
}

void TL2widget::changeBusy(void) {
  if(benabled->isChecked()) module->setbusy(1);
  else module->setbusy(0);
}

void TL2widget::loadFile(void)
{
  char *names[16];
  QString fileName = QFileDialog::getOpenFileName(this,
     "Load TL2-50 configuration file", "/home/roger", "Config Files (*.cfg)");

  for(int i=0; i<16; i++) names[i]=(char *)calloc(80,1);

  if (fileName!=NULL) {
    printf("Loading TL2-50 configuration from file %s\n",(const char *)fileName.toLocal8Bit().data());
    module->load((const char *)fileName.toLocal8Bit().data(),(char **)names);
    for(int i=0; i<16; i++) {
      inputs[i]->setText(QString(names[i]));
    }
  }
  
  for(int i=0; i<16; i++) free(names[i]);
}

void TL2widget::saveFile(void)
{
  char *names[16];

  QString fileName = QFileDialog::getSaveFileName(this, 
     "Save TL2-50 configuration file", "/home/roger", "Config Files (*.cfg)");

  for(int i=0; i<16; i++) names[i]=strdup(inputs[i]->text().toLocal8Bit().data());

  if (fileName!=NULL) {
    printf("Saving TL2-50 configuration to file %s\n",fileName.toLocal8Bit().data());
    module->save(fileName.toLocal8Bit().data(),names);
  }

  for(int i=0; i<16; i++) free(names[i]);  
}
