// tl2.cc - TL2-50 SUMMA trigger module test
//
// Oct-2008 by Solo

#include <stdio.h>
#include <string.h>
#include "tl2.h"

int main(int argc, char **argv)
{
  char qname[80];
  int crate=4;
  int module=6;
  int chan;
  unsigned int mask;

  if(argc<5) {
    printf("Usage: tl2 qbus crate module cmd \n");
    printf("       qbus   = 0-2  - qbus number or /dev/xxx \n");
    printf("       crate  = 0-15 - summa crate number \n");
    printf("       module = 1-21 - summa module number \n");
    printf("       cmd    = print         - print formatted contents of TL2 module \n");
    printf("              = mask [N][M]   - read/write mask M for channel N [0-3] \n");
    printf("              = trig [mask]   - read/write 4bit trigger OR mask \n");
    printf("              = psc [n][p]    - read/write 8bit prescaler n [0-1]\n");
    printf("              = counter [n]   - read 28bit input counter(s) [0-15]\n");
    printf("              = trigger [n]   - read 28bit trigger/busy counter(s) [0-4]\n");
    printf("              = freq [f]      - read/write intensimeter frequency divider [0-3]\n");
    printf("              = int           - read 2k words from 16bit intensity meter\n");
    printf("              = reset         - reset busy,intensimeter and all counters \n");
    printf("              = busy [enable] - read and enable=1/disable=0 autobusy \n");
    printf("              = save file     - save configuration to file \n");
    printf("              = load file     - load configuration from file \n");
    return 0;
  }

  strncpy(qname,argv[1],sizeof(qname));
  if(argv[1][0]=='0') strcpy(qname,"/dev/qbus0");
  if(argv[1][0]=='1') strcpy(qname,"/dev/qbus1");
  if(argv[1][0]=='2') strcpy(qname,"/dev/qbus2");

  sscanf(argv[2],"%d",&crate);
  if(crate<0 || crate>15) { 
    printf("SUMMA crate %d is out of [0-15] range\n",crate); 
    return -1;
  }

  sscanf(argv[3],"%d",&module);
  if(module<1 || module>21) { 
    printf("SUMMA module %d is out of [1-21] range\n",module); 
    return -1;
  }

  QBUS qb(qname);
  if(qb.node()<0) return -1;

  LE20C su(&qb, crate);

  TL2 tl(&su, module);

  // PRINT
  if (strcmp(argv[4],"print")==0) {
    tl.print();
  }
  // RESET
  else if (strcmp(argv[4],"reset")==0) {
    tl.reset();
  }
  // INT
  else if (strcmp(argv[4],"int")==0) {
    printf("intensity meter:\n");
    for(int i=0; i<2048; i++) {
      printf("int[%d]=%d\n",i,tl.intensity());
    }
  } 
  // MASK
  else if(strcmp(argv[4],"mask")==0) {
    if(argc<6) {
      printf("Usage: tl2 qbus crate module mask N [M]\n");
      return -1;
    }
    sscanf(argv[5],"%i",&chan);
    if(argc<7) {
      printf("chan %d mask = 0x%08x\n",chan,tl.readmask(chan));
    }
    else {
      sscanf(argv[6],"%x",&mask);
      tl.writemask(chan,mask);
    }
  }
  // TRIG
  else if(strcmp(argv[4],"trig")==0) {
    if(argc>5) {
      sscanf(argv[5],"%i",&mask);
      tl.writetrigmask(mask);
    }
    else printf("trigger mask = 0x%x \n",tl.readtrigmask());
  }
  // PRESCALER
  else if(strcmp(argv[4],"psc")==0) {
    if(argc<6) {
      printf("Usage: tl2 qbus crate module psc n [p]\n");
      return -1;
    }

    sscanf(argv[5],"%i",&chan);

    if(argc>6) {
      sscanf(argv[6],"%i",&mask);
      tl.writepsc(chan,mask);
    }
    else {
      printf("prescale %d = %d\n",chan, tl.readpsc(chan));
    }
  }
  // COUNTER
  else if(strcmp(argv[4],"counter")==0) {
    if(argc>5) {
      sscanf(argv[5],"%i",&chan);
      printf("counter[%d]=%d\n",chan,tl.counter(chan));
    }
    else {
      for(int i=0; i<16; i++) {
	printf("counter[%d]=%d\n",i,tl.counter(i));
      }
    }
  }
  // TRIGGER
  else if(strcmp(argv[4],"trigger")==0) {
    if(argc>5) {
      sscanf(argv[5],"%i",&chan);
      printf("trigger[%d]=%d %d\n",chan,tl.trigall(chan),tl.trigbusy(chan));
    }
    else {
      for(int i=0; i<4; i++) {
	printf("trigger[%d]=%d %d\n",i,tl.trigall(i),tl.trigbusy(i));
      }
    }
  }  
  // BUSY
  else if(strcmp(argv[4],"busy")==0) {
    if(argc>5) {
      sscanf(argv[5],"%i",&mask);
      tl.setbusy(mask);
    }
    else printf("busy = 0x%x \n",tl.readbusy());
  }
  // FREQ
  else if(strcmp(argv[4],"freq")==0) {
    if(argc>5) {
      sscanf(argv[5],"%i",&mask);
      tl.writefreq(mask);
    }
    else printf("freq = 0x%x \n",tl.readfreq());
  }
  // SAVE
  else if(strcmp(argv[4],"save")==0) {
    if(argc<6) {
      printf("Usage: tl2 qbus crate module save file\n");
      return -1;
    }
    tl.save(argv[5]);
  }
  else if(strcmp(argv[4],"load")==0) {
    if(argc<6) {
      printf("Usage: tl2 qbus crate module load file\n");
      return -1;
    }
    tl.load(argv[5]);
  }
  // UNKNOWN
  else {
    printf("unknown command: %s\n",argv[4]);
    return -1;
  }

  return 0;
}
