// go.h - GAMS movement 
//
// Nov-2004 by Solo, adapted from Serge 
// Nov-2007 by Solo, change in retry logic
// 18-Apr-2009 Remove unneeded move logic
// WARNING ! Now we can't read X coord while GAMS moving


#ifndef _GO_H
#define _GO_H


#include <math.h>   // pow
#include <stdlib.h> // abs
#include <unistd.h> // usleep
#include <stdint.h>

#include "le20c.h" // QBUS-SUMMA host
/////// /////
#include <time.h>
#include <sys/time.h>
//////  /////

// 0.092 or 3/32 - mm/summa
// 8.35          - summa/datchik
// 0.77          - mm/datchik

#define XY_DISABLE 0
#define X_ENABLE   1
#define Y_ENABLE   2
#define XY_ENABLE  3

// X, Y coordinates
typedef struct
{
  int X, Y;
} XY;

// Counter coordinates
typedef struct
{
  int column;
  int row;
  int quarter;
} CRQ;

//const int GO_X0 = -756+290; // up to run33
//const int GO_X0 = -1770;      // from run34, new beam line
//int GO_X0 ;
// = -1410; // from run35, after X encoder repair

//const int GO_Y0 = 21015; // up to run35
//int GO_Y0 ;
// = 20835; // from run35, new beam line 


const float GO_dX[5] = { 0, -21.5, -21.5, 21.5, 21.5 };
const float GO_dY[5] = { 0, 21.5, -21.5, 21.5, -21.5 };

//const int GO_miss_X = 10; // encoder units
//const int GO_miss_Y = 30; // encoder units
const int GO_miss_X = 30; // encoder units, run35
const int GO_miss_Y = 80; // encoder units, run35

const int GO_timeout    = 120;   // s
const int GO_stick_wait = 1;    // s

struct tm mytime;
time_t ltime;
char toDimMessage[256]="";
int is_all_ok=1;
char make_Mess[256]="";
class GO
{
 public:
  // constructor
  GO(LE20C* host,int nx=6, int ny=5, FILE* file=0)
  {
    h = host;
    X_n = nx;
    Y_n = ny;
    f = file;
   // f2= file2;
  }

  // destructor
  ~GO() {}

  // useful methods

  // convert Column/Row/Quarter to X/Y
  void crq2xy (CRQ* from, XY* to) {
    to->X = (int)((86.0*from->column + GO_dX[from->quarter])*32.0/3.0) + GO_X0;
    to->Y = (int)((-86.0*from->row + GO_dY[from->quarter])*32.0/3.0) + GO_Y0;
  }

  // convert X/Y to Column/Row/Quarter
  void xy2crq (XY* from, CRQ* to) {
    double D;
    double minD = 1.0e6;
    int minQ=0;
    XY approx;
    
    // Calculating nearest colomn and row
    to->column = (int)(((from->X - GO_X0)*3.0/32.0)/86.0 + 0.5);
    to->row = (int)(0.5 - ((from->Y - GO_Y0)*3.0/32.0)/86.0);

    // Looking for the nearest quarter
    for (to->quarter = 0; to->quarter < 5; to->quarter++) {
      crq2xy (to, &approx);
      D = pow ((float)approx.X - (float)from->X, 2) +
	pow ((float)approx.Y - (float)from->Y, 2);
      if (D < minD) {
	minD = D;
	minQ = to->quarter;
      }
    } 

    to->quarter = minQ;
  };

  /// get/set coordinates
  void set_X (short coord) 
  { short coordin;
  coordin = (short)(coord/0.964197531);
  h->write (X_n, 0, coordin); };
  void set_Y (short coord) { h->write (Y_n, 0, coord); };
  int  get_X () { return ((int)((0.964197531)*(h->function (X_n, 1, 2)))); };


 int  get_Y () { return  h->function (Y_n, 1, 2); };
 void get_XY (XY* pos) { pos->X = get_X(); pos->Y = get_Y(); };


  int64_t get_mks_time() {
    struct timeval tv;
    struct timezone tz;
    gettimeofday( &tv,&tz );
    return ((int64_t)1000000)*tv.tv_sec + tv.tv_usec;
  }


  // return true if moving
  int X_moving () { return (h->function (X_n, 2, 1) & 0x8000) == 0; };
  int Y_moving () { return (h->function (Y_n, 2, 1) & 0x8000) == 0; };


 

  // print current status

  void print () {
    const char* str[2] = { "no action", "moving" };
    XY xy;
    CRQ crq;

    get_XY (&xy);
    xy2crq (&xy, &crq);

//        printf ("GO status:\n");
//  printf("GO_X0= %d\nGO_Y0= %d \n",GO_X0,GO_Y0);
    printf ("X axis: status = %s, coord = %d\n", str[X_moving ()], xy.X);
    printf ("Y axis: status = %s, coord = %d\n", str[Y_moving ()], xy.Y);
    printf ("Closest position: column = %d, row = %d, quarter = %d\n",
	    crq.column, crq.row, crq.quarter);
  };
  //////////////////////// my //// get new position
//  XY posit() {
//    XY ddd;
//    ddd.X=get_X();
//    ddd.Y=get_Y();
//    return ddd;
//  }; 


  // move GAMS to new position
  int move (XY* dest, int ax=XY_ENABLE) {
    short last_coord;
    int s;
    is_all_ok=1;
// ltime=time(NULL);
localtime_r(&ltime,&mytime);
printf (  "\n\t\t%02d/%02d/%d - %02d:%02d:%02d\n",
        mytime.tm_mday,
        mytime.tm_mon+1,
        1900+mytime.tm_year,
        mytime.tm_hour,
        mytime.tm_min,
        mytime.tm_sec );
fprintf(f,"\n\t\t%02d/%02d/%d - %02d:%02d:%02d\n",
        mytime.tm_mday,
        mytime.tm_mon+1,
        1900+mytime.tm_year,
        mytime.tm_hour,
        mytime.tm_min,
        mytime.tm_sec );


    // start X movement
    // Can't measure X coordinate while moving
    if ((ax & X_ENABLE) != 0) {

      if (abs (get_X () - dest->X) > GO_miss_X)
	{
          int curr_coord;
	  last_coord = get_X ();
          fprintf (f,"Moving X from %d to %d...\n", last_coord, dest->X);
	  printf ("Moving X from %d to %d...\n", last_coord, dest->X);
          set_X (dest->X);
	  sleep (3*GO_stick_wait);
	  curr_coord = get_X();
          if (abs (curr_coord - dest->X) >= abs (last_coord - dest->X))
	    {
	      is_all_ok=0;
	      snprintf(toDimMessage, 255,"Alas: X movement stuck\n");
	      printf (  "Alas: X movement stuck, get_X=%d dest=%d\n",curr_coord,dest->X);
	      fprintf(f,"Alas: X movement stuck, get_X=%d dest=%d\n",curr_coord,dest->X);
	      h->Z ();
	    }
        }
      else { fprintf (f,"X coordinate is OK\n");
	     printf ("X coordinate is OK\n");
	   } 
   }

    // start Y movement
    if ((ax & Y_ENABLE) != 0)
      {
	if (abs (get_Y () - dest->Y) > GO_miss_Y)
	  {
            int curr_coord;
	    last_coord = get_Y ();
	    fprintf (f,"Moving Y from %d to %d...\n", last_coord, dest->Y);
	    printf ("Moving Y from %d to %d...\n", last_coord, dest->Y);
	    set_Y (dest->Y);
	    sleep (GO_stick_wait*3);
	    curr_coord = get_Y();
	    if (abs (curr_coord - dest->Y) >= abs (last_coord - dest->Y))
	      { 
		is_all_ok=0;
	        printf (  "Alas: Y movement stuck, get_Y=%d dest=%d\n",curr_coord,dest->Y);
	        fprintf(f,"Alas: Y movement stuck, get_Y=%d dest=%d\n",curr_coord,dest->Y);
		snprintf(toDimMessage,255,"Alas: Y movement stuck\n");
		h->Z ();
	      }
	 
	  }
	else   fprintf (f,"Y coordinate is OK\n");
                printf ("Y coordinate is OK\n");
	      
      }

  // now we are moving
#if 1
  s=0;
  while (X_moving() or Y_moving()) {
//  print();
    sleep(GO_stick_wait);

    if (s++ > GO_timeout) {
      printf ("Alas: timeout expired\n");
      print();
      h->Z ();
      return false;
    }
  }
#endif

sleep(GO_stick_wait);
fprintf(f,"Operation complete\n");
printf("Operation complete\n");

  if (((ax & X_ENABLE) != 0) and (abs (get_X () - dest->X) > GO_miss_X))
    {
     is_all_ok=0;
     fprintf (f,"Alas: X movement missed\n");
     printf ("Alas: X movement missed\n");
    }
  if (((ax & Y_ENABLE) != 0) and (abs (get_Y () - dest->Y) > GO_miss_Y))
    {
     is_all_ok=0;
     fprintf (f,"Alas: Y movement missed\n");
     printf ("Alas: Y movement missed\n");
//     strcat(make_Mess,"Alas: Y movement missed\n");
//     snprintf(toDimMessage,255, "Alas: Y movement missed\n");
    }



 // h->Z ();
    fprintf (f,"X  coord = %d\n",get_X());
    printf ("X  coord = %d\n",get_X());
    fprintf (f,"Y  coord = %d\n",get_Y());
    printf ("Y  coord = %d\n",get_Y());

  if (is_all_ok==1)
  { 
    XY xy; 
    CRQ crq;
    get_XY (&xy);
    xy2crq (&xy, &crq);

 snprintf(toDimMessage,255, "All is ok. last coord: X= %d\tY= %d\t\tCRQ=%02d%02d%d\n",get_X(),get_Y(), crq.column, crq.row, crq.quarter);

  }


  fflush(f);
  print();
  return true;
};

// move to destination coordinates
int move(CRQ *dest, int ax=XY_ENABLE) {
  XY dest_xy;
  crq2xy(dest, &dest_xy);
  return move(&dest_xy, ax);
}

// stop the movement
int stop() {
  h->Z();
  return 0;
}

private:
LE20C *h; // SUMMA host
int X_n;  // X module number
int Y_n;  // Y module number
FILE *f;
//FILE *f2;
};

#endif
