// le24.h - LE24, LE66 and LE68 MISS ADC controller 
//
// Mar-2004 by Solo, adapted from Serge
//
#ifndef _LE24_H
#define _LE24_H

#include "miss.h"

#define LE24_MAX_MODULE   18
#define LE24_MAX_CHANNEL  16
#define LE24_MAX_CHANNELS (LE24_MAX_MODULE*LE24_MAX_CHANNEL)
#define LE24_MAX_PED      4096

class LE24
{

 private:

  MISS* h;   // MISS host
  int ctrl;  // controller position (left or right)
  int first; // first LE19 module number in sector
  int last;  // last LE19 module number in sector

 public:

  enum controller {
    left  = 0x20, // Left controller, usually modules 10-18
    right = 0x21  // Right controller, usually modules 1-9
  };

  LE24 (MISS* host, int from, int to, int pos=0)
  {
    h = host;
    first = from;
    last = to;
    if (pos!=left && pos!=right) pos=first<10?right:left;
    ctrl=pos;
  }

  ~LE24 () {};

  int from()  { return first; }
  int to()    { return last;    }
  int pos()   { return ctrl;  }
  int shift() { return (first-1)*LE24_MAX_CHANNEL; }

  // read pedestals from controller memory
  int read (short *peds) {
    int c=0;

    h->clear(); // clear AR mode and reset bus
    for(int m=first; m<=last; m++) {
      for(int i=0; i<LE24_MAX_CHANNEL; i++) {
	h->write(ctrl,0,i+(m<<6));
	peds[c++]=h->read(ctrl,0);
      }
    }

    return c;
  }

  // write pedestals to controller memory
  int write (short *peds) {
    int c=0;

    h->clear(); // clear AR mode and reset bus
    for(int m=first; m<=last; m++) {
      for(int i=0; i<LE24_MAX_CHANNEL; i++) {
	h->write(ctrl,0,i+(m<<6));
	h->function(ctrl,0,5,peds[c++]); // MISS function 5 to write pedestal
      }
    }

    return c;
  }

  // refresh pedestals in controller memory
  int refresh (short *peds) {
    int c=0;
    int r=0;
    short p;

    h->clear(); // clear AR mode and reset bus
    for(int m=first; m<=last; m++) {
      for(int i=0; i<LE24_MAX_CHANNEL; i++) {
	h->write(ctrl,0,i+(m<<6));
	p=h->read(ctrl,0); // read pedestal
	if( peds[c]!=p ) {
	  h->write(ctrl,0,i+(m<<6));
	  h->function(ctrl,0,5,peds[c]); // MISS function 5 to write pedestal
	  r++;
	}
	c++;
      }
    }

    return r;
  }

};

# endif
