// le52.h - LE52 MISS timer by Nikolai Shalanda 
//
// Mar-2004 by Solo, adapted from Serge
//
#ifndef _LE52_H
#define _LE52_H

#include "miss.h"

typedef struct
{
  short a[8];
  short b[5];
  short c[8];
} LE52DATA;

class LE52
{

 public:

  LE52 (MISS* host, int number)
  {
    n = number;
    h = host;
  }

  ~LE52 () {};

  void write  (LE52DATA* data) {
    int cnt;
    for (cnt = 0; cnt < 8; cnt++)  h->write (n, cnt*4, data->a[cnt]);
    for (cnt = 0; cnt < 5; cnt++)  h->write (n, cnt*4 + 1, data->b[cnt]);
    for (cnt = 0; cnt < 8; cnt++)  h->write (n, cnt*4 + 2, data->c[cnt]);
  }

  void read   (LE52DATA* data) {
    int cnt;
    for (cnt = 0; cnt < 8; cnt++)  data->a[cnt] = h->read (n, cnt*4);
    for (cnt = 0; cnt < 5; cnt++)  data->b[cnt] = h->read (n, cnt*4 + 1);
    for (cnt = 0; cnt < 8; cnt++)  data->c[cnt] = h->read (n, cnt*4 + 2);
  }

  int  verify (LE52DATA* data) {
    LE52DATA rdata;
    int cnt;
    read (&rdata);

    int result = true;
    for (cnt = 0; cnt < 8; cnt++)
      if (rdata.a[cnt] != data->a[cnt]) result = false;
    for (cnt = 0; cnt < 5; cnt++)
      if ((rdata.b[cnt] != data->b[cnt]) and
	  (reversed (rdata.b[cnt]) != data->b[cnt])) result = false;
    for (cnt = 0; cnt < 8; cnt++)
      if (rdata.c[cnt] != data->c[cnt]) result = false;

    return result;
  }

  void print  () {
    LE52DATA rdata;
    int cnt;
    read (&rdata);

    printf("LE52 module %d contents:\nA:",n);
    for (cnt = 0; cnt < 8; cnt++) printf(" %d",rdata.a[cnt]);
    printf("\nB:");
    for (cnt = 0; cnt < 5; cnt++) printf(" %d",rdata.b[cnt]);
    printf("\nC:");
    for (cnt = 0; cnt < 8; cnt++) printf(" %d",rdata.c[cnt]);
    printf("\n");
  }

 private:
  MISS* h;
  int n;

  short reversed (short d)
    {
      return (d & 0xff00) + ((d & 0xf0) >> 4) + ((d & 0xf) << 4);
    }

};

# endif

