// le24.cc - test for LE24 MISS ADC controller
//
//  Mar-2004 by Solo
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>
#include "le20i.h"
#include "le20c.h"
#include "le24.h"
#include "pulse.h"

// print pedestals on the terminal
void print_data(short *data, int from, int to) {
  int c=0;
  char tag='0';

  printf("\n");
  printf("M\\C | 01 | 02 | 03 | 04 | 05 | 06 | 07 | 08 |");
  printf(      " 09 | 10 | 11 | 12 | 13 | 14 | 15 | 16 |\n");
  printf( "----+----+----+----+----+----+----+----+----+");
  printf(      "----+----+----+----+----+----+----+----+\n");
  
  for(int m=from; m<=to; m++) {
    printf(" %02d |",m);
    for(int i=0; i<16; i++) {
      tag='0'; // norm
      if(data[c]<100) tag='7';  // reverse - too low
      if(data[c]>300) tag='1';  // bold - high
      if(data[c]>1000) tag='5'; // blink - too high
      printf("\033[%cm%4d\033[0m ",tag,data[c++]);
    }
    printf("\n");
  }      
}

// print rms
void printf_data(float *data, int from, int to) {
  int c=0;
  char tag='0';

  printf("\n");
  printf("M\\C | 01 | 02 | 03 | 04 | 05 | 06 | 07 | 08 |");
  printf(      " 09 | 10 | 11 | 12 | 13 | 14 | 15 | 16 |\n");
  printf( "----+----+----+----+----+----+----+----+----+");
  printf(      "----+----+----+----+----+----+----+----+\n");
  
  for(int m=from; m<=to; m++) {
    printf(" %02d |",m);
    for(int i=0; i<16; i++) {
      tag='0'; // norm
      if(data[c]<0.01) tag='7';  // reverse - too low
      if(data[c]>3.0) tag='1';  // bold - high
      if(data[c]>10.0) tag='5'; // blink - too high
      printf("\033[%cm%4.1f\033[0m ",tag,data[c++]);
    }
    printf("\n");
  }      
}

int main(int argc, char **argv)
{
  char qname[80];
  char cmd[80]={"read"};
  int crate=1;
  int num=1;
  int pos=LE24::right;
  int from=1;
  int to=9;
  int fill=-1;
  int n=0;
  int cycles=1;

  short peds_read[LE24_MAX_CHANNELS];
  short peds_write[LE24_MAX_CHANNELS];
  short nums_read[LE24_MAX_CHANNELS];

  float peds;
  long peds_sum[LE24_MAX_CHANNELS];
  long rms_sum[LE24_MAX_CHANNELS];
  float rms_write[LE24_MAX_CHANNELS];

  if(argc<3) {
    printf("Usage: le24 qbus crate [pos] [from to] [cmd] \n");
    printf("       qbus  = 0-1  - qbus number or /dev/xxx \n");
    printf("       crate = 0-15 - miss crate number \n");
    printf("       pos   = 0-1  - ctrl position left-right \n");
    printf("       from  = 1-20 - first ADC module number \n");
    printf("       to    = 1-20 - last ADC module number \n");
    printf("       cmd   = read       - read pedestal memory\n");
    printf("               fill [ped] - fill pedestal memory with ped \n");
    printf("               test       - test pedestal memory \n");
    printf("               sdata [cycles] - read data serially (PCHI) \n");
    printf("               tnum  [cycles] - test module numbers \n");

    return 0;
  }

  strncpy(qname,argv[1],strlen(qname));
  if(argv[1][0]=='0') strcpy(qname,"/dev/qbus0");
  if(argv[1][0]=='1') strcpy(qname,"/dev/qbus1");

  sscanf(argv[2],"%d",&crate);
  if(crate<0 || crate>15) { printf("miss crate %d is out of [0-15] range\n",crate); return -1; }

  if(argc>3) {
    sscanf(argv[3],"%d",&num);
    if(num<0 || num>1) { 
      printf("LE24 controller number %d is out of [0-1] range\n",num); 
      return -1;
    }
    if(num==1) { pos=LE24::right; from=1; to=9; }
    else { pos=LE24::left; from=10; to=18; }
  }

  if(argc>5) {
    sscanf(argv[4],"%d",&from);
    if(from<1 || from>20) { printf("module from %d is out of [1-20] range!\n",from); return -1; }
    sscanf(argv[5],"%d",&to);
    if(to<1 || to>20) { printf("module to %d is out of [1-20] range!\n",to); return -1; }
  }

  if(argc>6) {
    sscanf(argv[6],"%s",cmd);
  }

  QBUS qb(qname);
  if(qb.node()<0) return -1;

  LE20I mi(&qb, crate);
  if(mi.status()<0) {
    printf("MISS crate %d not found!\n",crate);
    return -1;
  }

  LE24 adc(&mi, from, to, pos);
  
  // READ
  if(strcmp(cmd,"read")==0) {
    printf("MISS crate %d ADC %s ped memory:\n",crate,num==1?"right":"left");
    adc.read(peds_read);
    print_data(peds_read, from, to);
  }
  // SDATA
  else if(strcmp(cmd,"sdata")==0) {   

    // define pulser for triggering
    QBUS qbp("/dev/qbus0"); 
    LE20C su(&qbp, 4); // crate 4
    PULSE pu(&su, 14); // number 14
       
    for(int i=0; i<LE24_MAX_CHANNELS; i++)
      { 
      peds_sum[i]=0; // clear sum
      rms_sum[i]=0;
      }

    if(argc>7) sscanf(argv[7],"%d",&cycles);

    for(int i=0; i<cycles; i++) {
      //      mi.sync(); // does not work with all LE20
      pu.ped(); // pulse ped trigger
      n=mi.sdata(LE24_MAX_CHANNELS,peds_read);
      if(n < ((to-from+1)*16) ) {
	printf("%d Serial data read error: read %d words out of %d!\n",i,n,(to-from+1)*16); 
      }
      else {
	// add sum
	for(int j=0; j<n; j++){
         peds_sum[j] = peds_sum[j]+peds_read[j];
         rms_sum[j] = rms_sum[j]+peds_read[j]*peds_read[j]; 
       	}
      }
      mi.clear();     
    }

    for(int i=0; i<n; i++) {
      peds=(float)(peds_sum[i])/cycles;
      peds_write[i]= (short)(peds);
      rms_write[i] = sqrt (fabs((float)(rms_sum[i])/cycles-peds*peds ));
    }

    if(n>0) {
      printf("MISS crate %d mean:\n",crate);
      print_data(peds_write, from, to);
      printf("\n");
      printf("MISS crate %d rms:\n",crate);
      printf_data(rms_write, from, to);
    }
  }
  // FILL
  else if(strcmp(cmd,"fill")==0) {
    if(argc>7) sscanf(argv[7],"%i",&fill);
    if(fill<0) for(int i=0; i<LE24_MAX_CHANNELS; i++) peds_write[i]=i;
    else for(int i=0; i<LE24_MAX_CHANNELS; i++) peds_write[i]=fill;
    adc.write(peds_write);
  }
  // TEST
  else if(strcmp(cmd,"test")==0) {
    printf("MISS crate %d ADC %s ped memory test\n",crate,num?"right":"left");
    for(int cycle=0; cycle<LE24_MAX_PED; cycle++) {
      if (cycle%10==0) { printf("%d\r",cycle); fflush(stdout); }
      for(int i=0; i<LE24_MAX_CHANNELS; i++) peds_write[i]=cycle;
      adc.write(peds_write);
      adc.read(peds_read);
      for(int i=0; i<(to-from+1)*16; i++) {
	if(peds_read[i]!=peds_write[i]) {
	  printf("FAIL chan=%d write=%d read=%d\n",i,peds_write[i],peds_read[i]);
	}
      }
    }
  }
  // TNUM
  else if(strcmp(cmd,"tnum")==0) {
    // define pulser for triggering
    QBUS qbp("/dev/qbus0"); 
    LE20C su(&qbp, 4); // crate 4
    PULSE pu(&su, 14); // number 14

    if(argc>7) sscanf(argv[7],"%d",&cycles);
    for(int cycle=0; cycle<cycles; cycle++) {
      if (cycle%10==0) { printf("%d\r",cycle); fflush(stdout); }
      pu.ped(); // pulse ped trigger
      n=mi.sdata(LE24_MAX_CHANNELS,peds_read,nums_read);
      if(n < ((to-from+1)*16) ) {
	printf("%d Serial data read error: read %d words out of %d!\n",
	       cycle,n,(to-from+1)*16); 
      }
      else {
	// test numbers
	for(int j=0; j<n; j++) {
	  short m=nums_read[j]>>6;
	  short c=nums_read[j]&63;
	  short mm=j/16+from;
	  short cc=j%16;
	  if ( m != mm ) printf("%d FAIL module %d instead of %d\n",
				cycle,m,mm);
	  if ( c != cc ) printf("%d FAIL module %d chan %d instead of %d\n",
				cycle,m,c,cc); 
       	}
      }
      mi.clear();     
    }
  }
 // UNKNOWN
  else {
    printf("unknown command: %s\n",cmd);
    return -1;
  }
  return 0;
}

