// le71.cc - test for LE71 96-chan MISS ADC board (replaces LE-19)
//
//  Nov-2006 by Solo
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <math.h>
//#include "le20i.h"
#include "le83.h"
#include "le71.h"

// print pedestals on the terminal
void print_data(short *data) {
  int c=0;
  char tag='0';

  printf("\n");
  printf("M\\C | 01 | 02 | 03 | 04 | 05 | 06 | 07 | 08 |");
  printf(      " 09 | 10 | 11 | 12 | 13 | 14 | 15 | 16 |\n");
  printf( "----+----+----+----+----+----+----+----+----+");
  printf(      "----+----+----+----+----+----+----+----+\n");
  
  for(int m=0; m<LE71_MAX_CHANNEL/16; m++) {
    printf(" %02d |",m);
    for(int i=0; i<16; i++) {
      //      tag='0'; // norm
      //      if(data[c]<100) tag='7';  // reverse - too low
      //      if(data[c]>300) tag='1';  // bold - high
      //      if(data[c]>1000) tag='5'; // blink - too high
      //      printf("\033[%cm%4d\033[0m ",tag,data[c++]);
      printf("%4d ",data[c++]);
    }
    printf("\n");
  }      
}

// print rms
void printf_data(float *data, int from, int to) {
  int c=0;
  char tag='0';

  printf("\n");
  printf("M\\C | 01 | 02 | 03 | 04 | 05 | 06 | 07 | 08 |");
  printf(      " 09 | 10 | 11 | 12 | 13 | 14 | 15 | 16 |\n");
  printf( "----+----+----+----+----+----+----+----+----+");
  printf(      "----+----+----+----+----+----+----+----+\n");
  
  for(int m=from; m<=to; m++) {
    printf(" %02d |",m);
    for(int i=0; i<16; i++) {
      tag='0'; // norm
      if(data[c]<0.01) tag='7';  // reverse - too low
      if(data[c]>3.0) tag='1';  // bold - high
      if(data[c]>10.0) tag='5'; // blink - too high
      printf("\033[%cm%4.1f\033[0m ",tag,data[c++]);
    }
    printf("\n");
  }      
}

int main(int argc, char **argv)
{
  char qname[80];
  char cmd[80]={"rp"};
  int crate=1;
  int num=1;
  int fill=-1;
  int n=0;
  int cycles=1;

  short peds_read[LE71_MAX_CHANNEL];
  short peds_write[LE71_MAX_CHANNEL];
  short nums_read[LE71_MAX_CHANNEL];

  float peds;
  long peds_sum[LE71_MAX_CHANNEL];
  long rms_sum[LE71_MAX_CHANNEL];
  float rms_write[LE71_MAX_CHANNEL];

  if(argc<3) {
    printf("Usage: le71 qbus crate [num] [cmd] \n");
    printf("       qbus  = 0-1  - qbus number or /dev/xxx \n");
    printf("       crate = 0-15 - miss crate number \n");
    printf("       num   = 1-3  - module number \n");
    printf("       cmd   = rp       - read pedestal memory\n");
    printf("               fp ped   - fill pedestal memory with ped \n");
    printf("               fp ped chan - fill pedestal memory for channel chan with ped \n");
    printf("               tp       - test pedestal memory \n");
    printf("               cp       - clear pedestal memory \n");
    printf("               rd       - read data memory\n");
    printf("               fd [d]   - fill data memory with d \n");
    printf("               td       - test data memory \n");
    printf("               cd       - clear data memory \n");
    printf("               ta       - test address register \n");

    return 0;
  }

  strncpy(qname,argv[1],sizeof(qname));
  if(argv[1][0]=='0') strcpy(qname,"/dev/qbus0");
  if(argv[1][0]=='1') strcpy(qname,"/dev/qbus1");

  sscanf(argv[2],"%d",&crate);
  if(crate<0 || crate>15) { printf("miss crate %d is out of [0-15] range\n",crate); return -1; }

  if(argc>3) {
    sscanf(argv[3],"%d",&num);
    if(num<0 || num>10) { 
      printf("LE71 module number %d is out of [0-10] range\n",num); 
      return -1;
    }
  }

  if(argc>4) {
    sscanf(argv[4],"%s",cmd);
  }

  QBUS qb(qname);
  if(qb.node()<0) return -1;

  qb.reset();

  LE83 mi(&qb, crate);
  if(mi.status()<0) {
    printf("MISS crate %d not found!\n",crate);
    return -1;
  }

  LE71 adc(&mi, num);
  
  // READ PED
  if(strcmp(cmd,"rp")==0) {
    printf("MISS crate %d ADC %d ped memory:\n",crate,num);
    adc.read(peds_read,1);
    print_data(peds_read);
  }
  // FILL PED
  else if(strcmp(cmd,"fp")==0) {
    int chan;
    if(argc>5) sscanf(argv[5],"%i",&fill);
    if(argc>6) 
    {
      sscanf(argv[6],"%i",&chan);
      adc.write_ped(fill,chan);
    }
    else
    { 
    if(fill<0) for(int i=0; i<LE71_MAX_CHANNEL; i++) peds_write[i]=i;
    else for(int i=0; i<LE71_MAX_CHANNEL; i++) peds_write[i]=fill;
    adc.write(peds_write,1);
    }
  }
  // CLEAR PED
  else if(strcmp(cmd,"cp")==0) {
    adc.clear(1);
  }
  // TEST PED
  else if(strcmp(cmd,"tp")==0) {
    printf("MISS crate %d ADC %d ped memory test\n",crate,num);
    for(int cycle=0; cycle<LE71_MAX_PED; cycle++) {
      if (cycle%10==0) { printf("%d\r",cycle); fflush(stdout); }
      for(int i=0; i<LE71_MAX_CHANNEL; i++) peds_write[i]=cycle;
      adc.write(peds_write,1);
      adc.read(peds_read,1);
      for(int i=0; i<LE71_MAX_CHANNEL; i++) {
	if(peds_read[i]!=peds_write[i]) {
	  printf("FAIL chan=%d write=%d read=%d\n",i,peds_write[i],peds_read[i]);
	}
      }
    }
  }
  // READ DATA
  else if(strcmp(cmd,"rd")==0) {
    printf("MISS crate %d ADC %d data memory:\n",crate,num);
    adc.read(peds_read,0);
    print_data(peds_read);
  }
  // FILL DATA
  else if(strcmp(cmd,"fd")==0) {
    if(argc>5) sscanf(argv[5],"%i",&fill);
    if(fill<0) for(int i=0; i<LE71_MAX_CHANNEL; i++) peds_write[i]=i;
    else for(int i=0; i<LE71_MAX_CHANNEL; i++) peds_write[i]=fill;
    adc.write(peds_write,0);
  }
  // CLEAR DATA
  else if(strcmp(cmd,"cd")==0) {
    adc.clear(0);
  }
  // TEST DATA
  else if(strcmp(cmd,"td")==0) {
    printf("MISS crate %d ADC %d data memory test\n",crate,num);
    for(int cycle=0; cycle<LE71_MAX_PED; cycle++) {
      if (cycle%10==0) { printf("%d\r",cycle); fflush(stdout); }
      for(int i=0; i<LE71_MAX_CHANNEL; i++) peds_write[i]=cycle;
      adc.write(peds_write,0);
      adc.read(peds_read,0);
      for(int i=0; i<LE71_MAX_CHANNEL; i++) {
	if(peds_read[i]!=peds_write[i]) {
	  printf("FAIL chan=%d write=%d read=%d\n",i,peds_write[i],peds_read[i]);
	}
      }
    }
  }
  // TEST ADDRESS
  else if(strcmp(cmd,"ta")==0) {
    printf("MISS crate %d ADC %d address register test\n",crate,num);
    for(int addr=0; addr<96; addr++) {
      adc.waddr(addr);
      int i = adc.raddr();
      //printf(" write=%d read=%d\n",addr, i);
      if(i != addr ) {
	printf("FAIL write=%d read=%d\n",addr, i);
      }
    }
  }
 // UNKNOWN
  else {
    printf("unknown command: %s\n",cmd);
    return -1;
  }
  return 0;
}

