// qbus.cc - test for qbus device
//
//  Mar-2004 by Solo
//  Dec-2006 by Solo - add clear command

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include "qbus.h"

int main(int argc, char **argv)
{
  char qname[80];
  unsigned int addr=0;
  unsigned int data=0;
  int s;
  struct timeval tv0, tv1;

  
  if(argc<3) {
    printf("Usage: qbus dev cmd \n");
    printf("       dev = 0,1 or /dev/xxx \n");
    printf("       cmd = reset      - reset QBUS branch \n");
    printf("             clear      - clear error status \n");
    printf("             read       - read word from QBUS \n");
    printf("             write word - write word to QBUS \n");
    return 0;
  }

  strncpy(qname,argv[1],sizeof(qname));
  if(argv[1][0]=='0') strcpy(qname,"/dev/qbus0");
  if(argv[1][0]=='1') strcpy(qname,"/dev/qbus1");

  QBUS qb(qname);
  if(qb.node()<0) return -1;

  // RESET
  if(strcmp(argv[2],"reset")==0) {
    qb.reset();
    return 0;
  }
  // CLEAR
  else if(strcmp(argv[2],"clear")==0) {
    qb.clear();
    return 0;
  }
  // READ
  else if(strcmp(argv[2],"read")==0) {
    if(argc<4) {
      printf("Usage: qbus dev read addr\n");
      return 0;
    }
    sscanf(argv[3],"%i",&addr);
    printf("read %s[0x%x]=0x%x\n",qname,addr,qb.read(addr));
    return 0;
  }
  // WRITE
  else if(strcmp(argv[2],"write")==0) {
    if(argc<5) {
      printf("Usage: qbus dev write addr data\n");
      return 0;
    }
    sscanf(argv[3],"%i",&addr);
    sscanf(argv[4],"%i",&data);
    qb.write(addr,data);
    printf("write %s[0x%x]=0x%x\n",qname,addr,data);
    return 0;
  }
  // TEST
  else if(strcmp(argv[2],"test")==0) {
    double usec;

    printf("Testing lseek()...");
    gettimeofday(&tv0,0); 
    for(int i=0; i<1000000; i++) qb.lseek(0,3);
    gettimeofday(&tv1,0);
    usec = (tv1.tv_sec-tv0.tv_sec)*1000000.0 + tv1.tv_usec-tv0.tv_usec;
    printf("%f usec/call\n",usec/1000000.0);

    printf("Testing clear()...");
    gettimeofday(&tv0,0); 
    for(int i=0; i<1000000; i++) qb.clear();
    gettimeofday(&tv1,0);
    usec = (tv1.tv_sec-tv0.tv_sec)*1000000.0 + tv1.tv_usec-tv0.tv_usec;
    printf("%f usec/call\n",usec/1000000.0);

    printf("Testing read()...");
    gettimeofday(&tv0,0);
    for(int i=0; i<1000000; i++) data=qb.read(0xff42);
    gettimeofday(&tv1,0);
    usec = (tv1.tv_sec-tv0.tv_sec)*1000000.0 + tv1.tv_usec-tv0.tv_usec;
    printf("%f usec/call\n",usec/1000000.0);
  }
  // UNKNOWN
  else {
    printf("unknown command: %s\n",argv[2]);
    return -1;
  }

  return 0;
}
