// tl2qt.cpp - Qt GUI interface for TL2-50 module
//
// Nov-2008 by Solo
// Apr-2009 by Solo - time display, lock file mechanism

#include <QApplication>
#include <QVBoxLayout>
#include <QPushButton>
#include <QTimer>
#include "tl2_widget.h"
#include "timer.h"

#include <sys/file.h>

int main(int argc, char *argv[])
{
  QBUS *qu;
  LE20C *su;
  TL2 *tl;

  // help string
  if (argc>1) {
    if (strcmp(argv[1],"-h")==0 || strcmp(argv[1],"-?")==0) {
      printf("Usage: tl2qt [module] [crate] [qbus]\n");
      printf("       where module=0-21, default=12\n");
      printf("             crate=4\n");
      printf("             qbus=/dev/pq2\n");
      exit(0);
    } 
  }
	
  // create hardware objects

  if (argc>3) {
    qu = new QBUS(argv[3]);
    su = new LE20C(qu,atoi(argv[2]));
    tl = new TL2(su,atoi(argv[1]));
  }
  else if (argc>2) {
    qu = new QBUS("/dev/pq2");
    su = new LE20C(qu,atoi(argv[2]));
    tl = new TL2(su,atoi(argv[1])); 
  }
  else if (argc>1) {
    qu = new QBUS("/dev/pq2");
    su = new LE20C(qu,4);
    tl = new TL2(su,atoi(argv[1]));
  }
  else {
    qu = new QBUS("/dev/pq2");
    su = new LE20C(qu,4);
    tl = new TL2(su,12);
  }

  // allow only one process to access one module

  char lockfilename[FILENAME_MAX];
  sprintf(lockfilename,"/tmp/tl2qt_%d_%d.lock",su->number(),tl->number());
  FILE *lockfile = fopen(lockfilename,"a+");
  if (lockfile==NULL) {
    printf("Cannot open lock file %s\n",lockfilename);
    printf("exiting...");
    exit(0);
  }

  if ( flock(fileno(lockfile),LOCK_EX|LOCK_NB)<0 ) {
    int pid=0;
    fscanf(lockfile,"%d",&pid);
    printf("TL2-50 at crate %d module %d is locked by another process id %d\n",su->number(),tl->number(),pid);
    printf("exiting...\n");
    exit(0);
  }

  fprintf(lockfile,"%d\n",getpid());
  fflush(lockfile);

  // create application
  QApplication app(argc, argv);
  QWidget win;
  QVBoxLayout vbox;
  Timer dt;
  TL2widget widget(tl);
  QPushButton quit("&Quit");

  // lay out stuff
  vbox.setContentsMargins(0,0,0,0);
  vbox.addWidget(&dt);
  vbox.addWidget(&widget);
  vbox.addWidget(&quit);
  win.setLayout(&vbox);

  // connect quit button
  QObject::connect(&quit, SIGNAL(clicked()), &app, SLOT(quit()));

  // create and connect update timer
  QTimer *timer = new QTimer(&app);
  QObject::connect(timer, SIGNAL(timeout()), &widget, SLOT(readTL2()));
  QObject::connect(timer, SIGNAL(timeout()), &dt, SLOT(updateTime()));
  timer->start(3000);

  // run the show
  win.show();
  app.exec();

  // remove the lock
  flock(fileno(lockfile),LOCK_UN);
  fclose(lockfile);
  remove(lockfilename);
}
